package salsa.language;

/**
 * Runtime class.
 *
 * Initially just keep a counter for unprocessed actor messages.
 * When there are no unprocessed actor messages and all actors are
 * local and are waiting for messages:  we can exit the Java 
 * virtual machine.
 */

public class RunTime {
	// Here we have a lower priority thread that checks for
	// conditions to terminate the JVM:
	// (1) All actors are local.  (no universal actors)
	// (2) All actor mailboxes are empty.
	// (3) No actor is currently processing a message.

	static Thread exitThread;
	static int messagesInMailboxes = 0;
	static int universalActors = 0;
	static boolean started = false;
	static boolean initialized = false;

	private static void init() {
		Runnable checkExit = new Runnable() {
			public void run() {
				while ((universalActors > 0) || (messagesInMailboxes > 0)) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException ie) {}
				}

				System.err.flush();
				System.out.flush();

				System.runFinalization();
				System.exit(0);
			}
		};
		
		exitThread = new Thread(checkExit);
		exitThread.setDaemon(true);
		exitThread.setPriority(Thread.MIN_PRIORITY);
		initialized = true;
	}

	protected synchronized static void receivedMessage() {
		if (!initialized) {
			init();
		}
		messagesInMailboxes++;
		if (!started) {
			started = true;
			exitThread.setName( "RunTime Exit Thead" );
			exitThread.start();
		}
	}

	protected synchronized static void finishedProcessingMessage() {
		messagesInMailboxes--;
	}

	// public for access to wwc.messaging package.
	public synchronized static void receivedUniversalActor() {
		if (universalActors == 0) ServiceFactory.getTheater().startTheater(null);
		universalActors++;
	}

	public synchronized static void removedUniversalActor() {
		universalActors--;
	}
}
